/*
 *  mlinput.c -- implements the multiline input box
 *
 *  AUTHOR: Christian Ordig (chr.ordig@gmx.net)
 *  !!!! This code is still in the test state !!!!
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <form.h>
#include "dialog.h"

unsigned char dialog_mlinput_result[MAX_LEN + 1];
unsigned char tmp[MAX_LEN + 1];

/*
 * Display a dialog box for inputing a string
 */
int
dialog_mlinput (const char *title, const char *cprompt, int height, int width, int numrows,
		 const char *init)
{
    
    int i, j, ij, xi, yi, x, y, box_y, box_x, box_width, box_height, additional_lines;
    int input_x = 0, key = 0, button = -1;
    WINDOW *dialog;
    WINDOW *w;
    FIELD *input;
    FIELD *felder[2];
    FORM *form;

    char *prompt=strclone(cprompt);

    tab_correct_str(prompt);
    if (init != NULL) {
      prompt=auto_size(prompt, &height, &width, 5, MIN(MAX(strlen(init)+7,26),SCOLS-(begin_set ? begin_x : 0)));
    } else
      prompt=auto_size(prompt, &height, &width, 5, 26);
    print_size(height, width);
    ctl_size(height, width);
  
    if (begin_set == 1) {
      x = begin_x;
      y = begin_y;
    } else {
      /* center dialog box on screen */
      x = (SCOLS - width)/2;
      y = (SLINES - height)/2;
    }

#ifdef HAVE_NCURSES
    if (use_shadow)
	draw_shadow (stdscr, y, x, height, width);
#endif
    dialog = newwin (height, width, y, x);
    mouse_setbase (x, y);
    keypad (dialog, TRUE);
    
    box_width = width - 6;
    box_height = height - 6;
    box_y = y + 2;
    box_x = (width - box_width) / 2;
    
    w=subwin (dialog,box_height,box_width,box_y,box_x+x);
    keypad (w, TRUE);
    immedok(w, TRUE);

    draw_box (dialog, 0, 0, height, width, dialog_attr, border_attr);
    wattrset (dialog, border_attr);
    wmove (dialog, height - 3, 0);
    waddch (dialog, ACS_LTEE);
    for (i = 0; i < width - 2; i++)
	waddch (dialog, ACS_HLINE);
    wattrset (dialog, dialog_attr);
    waddch (dialog, ACS_RTEE);
    wmove (dialog, height - 2, 1);
    for (i = 0; i < width - 2; i++)
	waddch (dialog, ' ');

    if (title != NULL) {
	wattrset (dialog, title_attr);
	wmove (dialog, 0, (width - strlen (title)) / 2 - 1);
	waddch (dialog, ' ');
	waddstr (dialog, title);
	waddch (dialog, ' ');
    }
    wattrset (dialog, dialog_attr);
    print_autowrap (dialog, prompt, width, 1, 2);

    /* Draw the input field box */
    
    mouse_mkregion (y + 1, box_x - 1, height-5, box_width + 2, 'i');

    
    
    additional_lines=numrows-box_height;
    if (additional_lines < 0) {additional_lines=0;}
    input=new_field(box_height,box_width,0,0,additional_lines,0);
    set_field_back(input,inputbox_attr);    
    set_field_buffer (input,0,init);
    set_field_opts(input, O_VISIBLE | O_ACTIVE | O_PUBLIC | O_EDIT | O_WRAP | O_STATIC);
//    set_field_opts(input, O_VISIBLE | O_ACTIVE | O_PUBLIC | O_EDIT | O_WRAP);
    felder[0]=input;
    felder[1]=(FIELD*) NULL;
        
    form=new_form(felder);
    set_form_win(form,w);
    post_form(form);
    
//    draw_box (dialog, y + 1, box_x - 1, height-5, box_width + 2,
//	      border_attr, dialog_attr);

    x = width / 2 - 11;
    y = height - 2;
    print_button (dialog, "Cancel", y, x + 14, FALSE);
    print_button (dialog, "  OK  ", y, x, FALSE);

    /* Set up the initial value */
    wmove (dialog, box_y, box_x);
    wattrset (dialog, inputbox_attr);
    wrefresh (dialog);
    raw();
    wmove (w,0,0);
    form_driver(form,REQ_NEW_LINE);
    form_driver(form,REQ_BEG_LINE);
    while (key != ESC) {
	key = wgetch (dialog);
	if (button == -1) {	/* Input box selected */
	    switch (key) {
	    case '\n':
		form_driver(form,REQ_NEW_LINE);
		continue;
	    case TAB:
		getyx (dialog,yi,xi);
	    	break;
	    case KEY_HOME:
		form_driver(form,REQ_BEG_LINE);
		continue;
	    case KEY_END:
		form_driver(form,REQ_END_LINE);
		continue;	
	    case KEY_UP:
		form_driver(form,REQ_UP_CHAR);
	    	continue;
	    case KEY_DOWN:
		form_driver(form,REQ_DOWN_CHAR);
	    	continue;
	    case M_EVENT + 'i':
	    case M_EVENT + 'o':
	    case M_EVENT + 'c':
		break;
	    case KEY_LEFT:
		form_driver(form,REQ_PREV_CHAR);
		continue;
	    case KEY_RIGHT:
		form_driver(form,REQ_NEXT_CHAR);
	    	continue;
	    case KEY_BACKSPACE:
	    case 127:
		form_driver(form,REQ_DEL_PREV);
		continue;
	    case KEY_DC:
		form_driver(form,REQ_DEL_CHAR);
		continue;
	    case KEY_DL:
		form_driver(form,REQ_DEL_LINE);
		continue;
	    default:
		if (key < 0x100 && isprint (key)) {
		    form_driver(form,key);
//		    wrefresh (w);
		    continue;
		}
	    }
	}
	switch (key) {
	case 'C':
	case 'c':
	    delwin (dialog);
	    delwin (w);
	    return 1;
	case M_EVENT + 'i':	/* mouse enter events */
	case M_EVENT + 'o':	/* use the code for 'UP' */
	case M_EVENT + 'c':
	    button = (key == M_EVENT + 'o') - (key == M_EVENT + 'c');
	case KEY_UP:
	case KEY_LEFT:
	    switch (button) {
	    case -1:
		button = 1;	/* Indicates "Cancel" button is selected */
		print_button (dialog, "  OK  ", y, x, FALSE);
		print_button (dialog, "Cancel", y, x + 14, TRUE);
		wrefresh (dialog);
		break;
	    case 0:
		button = -1;	/* Indicates input box is selected */
		print_button (dialog, "Cancel", y, x + 14, FALSE);
		print_button (dialog, "  OK  ", y, x, TRUE);
		wmove (dialog, box_y, box_x + input_x);
		wrefresh (dialog);
		break;
	    case 1:
		button = 0;	/* Indicates "OK" button is selected */
		print_button (dialog, "Cancel", y, x + 14, FALSE);
		print_button (dialog, "  OK  ", y, x, TRUE);
		wrefresh (dialog);
		break;
	    }
	    break;
	case TAB:
	case KEY_DOWN:
	case KEY_RIGHT:
	    switch (button) {
	    case -1:
		button = 0;	/* Indicates "OK" button is selected */
		print_button (dialog, "Cancel", y, x + 14, FALSE);
		print_button (dialog, "  OK  ", y, x, TRUE);
		wrefresh (dialog);
		break;
	    case 0:
		button = 1;	/* Indicates "Cancel" button is selected */
		print_button (dialog, "  OK  ", y, x, FALSE);
		print_button (dialog, "Cancel", y, x + 14, TRUE);
		wrefresh (dialog);
		break;
	    case 1:
		button = -1;	/* Indicates input box is selected */
		print_button (dialog, "Cancel", y, x + 14, FALSE);
		print_button (dialog, "  OK  ", y, x, FALSE);
		wmove (dialog,yi,xi);
		wrefresh (dialog);
		break;
	    }
	    break;
	case 'O':
	case 'o':
	    button=-1;
	case ' ':
	case '\n':
	    form_driver(form,REQ_END_FIELD);	
	    form_driver(form,REQ_NEW_LINE);
	    strcpy (tmp,field_buffer(input,0));
	    ij=0;
	    if (numrows<box_height) numrows=box_height;
	    for (i=1;i<=numrows;i++) {
		for (j=1;j<=box_width;j++) {
		    dialog_mlinput_result[ij]=tmp[ij-(i-1)];ij++;
		}
		dialog_mlinput_result[ij]='\n';ij++;
	    }
	    dialog_mlinput_result[ij]='\0';
	    delwin (dialog);
	    delwin (w);
	    return (button == -1 ? 0 : button);
	case ESC:
	    break;
	}
    }
    delwin (w);
    delwin (dialog);
    return -1;			/* ESC pressed */
}
